//
// The MIT License (MIT)
//
// Copyright (c) 2013 Backelite
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <Foundation/Foundation.h>

#define BkAbstractMethodAssert() NSAssert2(NO, @"%@ is an abstract method and must be overriden in '%@'", NSStringFromSelector(_cmd), [self class])
#define BkSimpleAssert(condition) NSAssert1(condition, @"Assert not satisfying: %s", #condition)
/**
 * a macro to easily detects when a deprecated method is used (use NSAssert)
 */
#define BkMethodDeprecated(format, ...) NSAssert(NO, (@"DEPRECATED: %s (used on <%@ %p>) is deprecated: " format), __PRETTY_FUNCTION__, [self class], self , ##__VA_ARGS__)

/**
 * Convenient Assert to test a parameter class
 */
#define BkParameterClassAssert(param, expectedClass)        NSParameterAssert(param != nil && [param isKindOfClass:[expectedClass class]])
/**
 * Convenient Assert to test a parameter class if not nil
 */
#define BkParameterClassOrNilAssert(param, expectedClass)   NSParameterAssert(param == nil || [param isKindOfClass:[expectedClass class]])
